#include "ScreenManager.h"

ScreenManager::ScreenManager(Adafruit_SSD1306 &display) : _displayRef(display), _displayColorInverted(false), _displayDimmed(false), _currentView(NULL), _error(0)
{  
  _viewLinkedList = (ViewLinkedList) createEmptyList();
  _displayRef.setRotation(OR_0);
  _displayRef.setTextColor(WHITE);
}

boolean ScreenManager::addView(boolean (*viewLogicFunction)(Adafruit_SSD1306&, void*), void *pData, const unsigned char UID)
{
  ViewLink viewLink ={viewLogicFunction, pData, UID, NULL};
  addNewLinkAtTheEnd(&_viewLinkedList, viewLink);
}

void *ScreenManager::createEmptyList()
{
  return NULL;
}

boolean ScreenManager::addNewLinkAtTheEnd(ViewLinkedList *viewLinkedList, ViewLink viewLink)
{
  ViewLink *newViewLink = (ViewLink*) malloc(sizeof(ViewLink));
  if(newViewLink == NULL)
  {
    _error |= MALLOC_ERR;
    return false;
  }
  
  //Because of the const member
  memcpy(newViewLink, &viewLink, sizeof(*newViewLink));

  if(isListEmpty(*viewLinkedList))*viewLinkedList = newViewLink;
  else
  {
    if((*viewLinkedList)->UID == newViewLink->UID)
    {
        ViewLink *link = *viewLinkedList;
        *viewLinkedList = newViewLink;
        newViewLink->next = link->next;
        free(link);
        return true;
    }
    
    ViewLinkedList cursor = *viewLinkedList;
    
    while(cursor->next != NULL)
    {
      if(cursor->next->UID == newViewLink->UID)
      {
        ViewLink *link = cursor->next;
        cursor->next = newViewLink;
        newViewLink->next = link->next;
        free(link);
        return true;
       }
      cursor = cursor->next;
    }
    cursor->next = newViewLink;
  }
  
  return true;
}

boolean ScreenManager::removeView(const unsigned char UID)
{
  return removeLinkByUID(&_viewLinkedList ,UID);
}

boolean ScreenManager::removeLinkByUID(ViewLinkedList *viewLinkedList, const unsigned char UID)
{
  if(isListEmpty(*viewLinkedList))return false;

  if((*viewLinkedList)->UID == UID)
  {
    ViewLink *tmp = *viewLinkedList;
    *viewLinkedList = (*viewLinkedList)->next;
    free(tmp);
    return true;
  }

  ViewLinkedList cursor = *viewLinkedList;
  while(!isListEmpty(cursor->next))
  {
    if(cursor->next->UID == UID)
    {
      ViewLink *tmp = cursor->next;
      cursor->next = cursor->next->next;
      free(tmp);
      return true;
     }
     cursor = cursor->next;
  }
  return false;
}

boolean ScreenManager::isListEmpty(ViewLinkedList viewLinkedList)
{
  return viewLinkedList == NULL;
}

unsigned char ScreenManager::getError() const
{
  return _error;
}

void ScreenManager::iterateThroughList()
{
  Serial.println("Lets go through");
  ViewLinkedList temp = _viewLinkedList;
  while(!isListEmpty(temp))
  {
    Serial.print("UID : ");Serial.println(temp->UID);
    temp = temp->next;
  }
}

ViewLink* ScreenManager::getLinkByUID(ViewLinkedList viewLinkedList, const unsigned char UID)
{
  if(isListEmpty(viewLinkedList))
    return NULL;

  while(!isListEmpty(viewLinkedList))
  {
    if(viewLinkedList->UID == UID)return viewLinkedList;
    viewLinkedList = viewLinkedList->next;
  }

  return NULL;
}

boolean ScreenManager::displayView(const int UID)
{
  boolean result = false;
  
  if(UID == -1 && _currentView == NO_CURRENT_VIEW)
  {
    //We display an error message on the screen
    
    return false;
  }
  else if(UID == -1)
  {
    _displayRef.clearDisplay();
    _displayRef.setCursor(0,0);
    result = (*_currentView->viewLogicFunction)(_displayRef, _currentView->pData);
    _displayRef.display();
    
    return result;
  }
    
  ViewLink *viewLink = getLinkByUID(_viewLinkedList, UID);
  if(viewLink == NULL)
  {
    //We display an error message on the screen
    
    return false;
  }else if(viewLink->viewLogicFunction == NULL)
  {
    //We display an error message on the screen
    
    return false;
  }
  
  _currentView = viewLink;

  _displayRef.clearDisplay();
  _displayRef.setCursor(0,0);
  result = (*viewLink->viewLogicFunction)(_displayRef, viewLink->pData);
  _displayRef.display();
  
  return result;
}

void ScreenManager::invertDisplayColor(const boolean inverted)
{
  _displayRef.invertDisplay(inverted);
  _displayColorInverted = inverted;
}

void ScreenManager::dimDisplay(const boolean dimmed)
{
  _displayRef.dim(dimmed);
  _displayDimmed = dimmed;
}

void ScreenManager::orientDisplay(const Orientation orientation)
{
  _displayRef.setRotation(orientation);
}

boolean ScreenManager::isDisplayColorInverted() const
{
  return _displayColorInverted;
}

Orientation ScreenManager::getDisplayOrientation() const
{
  return (Orientation) _displayRef.getRotation();
}

boolean ScreenManager::isDisplayDimmed() const
{
  return _displayDimmed;
}

void ScreenManager::clearDisplay(const boolean bufferOnly)
{
  if(bufferOnly)
    _displayRef.clearDisplay();
  else
  {
    _displayRef.clearDisplay();
    _displayRef.display();
  }
}


